/*
  Copyright (C) 2012
  Alejandro Mujica (amujica@cenditel.gob.ve)
  Erwin Paredes (eparedes@cenditel.gob.ve)
  José Ruiz (jruiz@cenditel.gob.ve)
  Rodolfo Rangel (rrangel@cenditel.gob.ve)
  Julie Vera (jvera@cenditel.gob.ve)

  CENDITEL Fundación Centro Nacional de Desarrollo e Investigación en
  Tecnologías Libres

  Este programa es software libre; Usted puede usarlo bajo los términos de la
  licencia de software GPL versión 2.0 de la Free Software Foundation.

  Este programa se distribuye con la esperanza de que sea útil, pero SIN
  NINGUNA GARANTÍA; tampoco las implícitas garantías de MERCANTILIDAD o
  ADECUACIÓN A UN PROPÓSITO PARTICULAR.
  Consulte la licencia GPL para más detalles. Usted debe recibir una copia
  de la GPL junto con este programa; si no, escriba a la Free Software
  Foundation Inc. 51 Franklin Street,5 Piso, Boston, MA 02110-1301, USA.
*/

/*
  Este archivo contiene la implementación de la clase TempFilterForm, la cual
  repesenta el formulario utilizado para filtrar los productos que serán las
  raíces de la red que se va a construir para estudiar.

  Autor: Alejandro J. Mujica
  Fecha: 04/12/2013
*/

# include <tempFilterForm.H>

# include <QMessageBox>

# include <ioManager.H>
# include <dbProperties.H>

# include <configuration.H>
# include <simType.H>

struct FillComboYear
{
    QComboBox * ptrComboYear;

    FillComboYear(QComboBox * _ptrComboYear)
        : ptrComboYear(_ptrComboYear) {
        // Empty
    }

    void operator () (const std::string & year, const unsigned short &) {
        ptrComboYear->addItem(year.c_str());
    }
};

void TempFilterForm::moveToCenter(QWidget * parent) {

    QPoint halfForm(int(width() / 2), int(height() / 2));

    QPoint halfParent(int(parent->width() / 2), int(parent->height() / 2));

    QPoint position = parent->pos() + halfParent - halfForm;

    move(position);
}

void TempFilterForm::updateComboYear() {

    ui.comboYear->clear();

    ui.comboYear->addItem("Seleccione Año");

    yearsMap.for_each<FillComboYear>(FillComboYear(ui.comboYear));
}

TempFilterForm::TempFilterForm(QWidget *parent)
    : QWidget(parent, Qt::Tool) {

    ui.setupUi(this);

    moveToCenter(parent);

    selectedItem = NULL;

    ui.btnAddProductToSelectedList->setEnabled(false);

    ui.btnRemoveProductFromSelectedList->setEnabled(false);

    ui.edtCompanyName->setFocus();

    ui.layoutWaitingPanel->addWidget(&waitingPanel, Qt::AlignCenter);

    connect(ui.btnAccept, SIGNAL(clicked()),
            this, SLOT(slotBtnAcceptClicked()));

    connect(ui.edtCompanyName, SIGNAL(textChanged(QString)),
            this, SLOT(slotTextChanged()));

    connect(ui.companyList, SIGNAL(itemClicked(QListWidgetItem*)),
            this, SLOT(slotCompanySelected(QListWidgetItem*)));

    connect(ui.productList, SIGNAL(itemClicked(QListWidgetItem*)),
            this, SLOT(slotProductSelected(QListWidgetItem*)));

    connect(ui.selectedProductList, SIGNAL(itemClicked(QListWidgetItem*)),
            this, SLOT(slotSelectedProductSelected(QListWidgetItem*)));

    connect(ui.btnAddProductToSelectedList, SIGNAL(clicked()),
            this, SLOT(slotBtnAddProductToSelectedListClicked()));

    connect(ui.btnRemoveProductFromSelectedList, SIGNAL(clicked()),
            this, SLOT(slotBtnRemoveProductFromSelectedList()));

    connect(&process, SIGNAL(finished(int)),
            this, SLOT(slotGraphBuilderExecuted()));

    ui.edtCompanyName->setFocus();
}

void TempFilterForm::slotBtnAcceptClicked() {

    if (ui.selectedProductList->count() == 0) {
        QMessageBox::critical(this, "Error",
                              "No se ha seleccionado ningún producto");

        ui.edtCompanyName->setFocus();

        return;
    }

    if (ui.comboStudyType->currentIndex() == 0) {
        QMessageBox::critical(this, "Error",
                              "Debe seleccionar un tipo de estudio");

        ui.comboStudyType->setFocus();

        return;
    }

    if (ui.comboYear->currentIndex() == 0) {
        QMessageBox::critical(this, "Error",
                              "Debe seleccionar un año de estudio");

        ui.comboYear->setFocus();

        return;
    }

    Configuration & conf = Configuration::getInstance();

    std::string rootsFileName = FILE_PATH(conf, Xml, Roots, xml).c_str();

    IOManager ioManager;

    DynDlist <long> productIdList;

    for (size_t i = 0; i < ui.selectedProductList->count(); ++i) {

        QListWidgetItem * item = ui.selectedProductList->item(i);

        productIdList.append(item->data(1).toString().toLong());
    }

    ioManager.createXmlRootsFile(ui.comboYear->currentText().toStdString(),
                                 productIdList, ui.spinUpstreamLevels->value(),
                                 ui.spinDownstreamLevels->value(),
                                 rootsFileName);

    conf.createDbFile();

    std::string graphXmlFileName = FILE_PATH(conf, Xml, Graph, xml).c_str();

    QStringList args;

    args.append(QString("-i") + rootsFileName.c_str());
    args.append(QString("-o") + graphXmlFileName.c_str());
    args.append(QString("-b") + conf.getDbFileName().c_str());

    process.start(conf.getGraphBuilderCommand().c_str(), args);

    waitingPanel.start();

    ui.btnAccept->setEnabled(false);
    ui.btnCancel->setEnabled(false);
    ui.btnAddProductToSelectedList->setEnabled(false);
    ui.btnRemoveProductFromSelectedList->setEnabled(false);
    ui.companyList->setEnabled(false);
    ui.productList->setEnabled(false);
    ui.selectedProductList->setEnabled(false);
    ui.comboStudyType->setEnabled(false);
    ui.comboYear->setEnabled(false);
    ui.spinDownstreamLevels->setEnabled(false);
    ui.spinUpstreamLevels->setEnabled(false);
}

void TempFilterForm::slotTextChanged() {

    if (selectedItem != NULL) {
        selectedItem->setSelected(false);
        selectedItem = NULL;
    }

    ui.btnAddProductToSelectedList->setEnabled(false);
    ui.btnRemoveProductFromSelectedList->setEnabled(false);
    ui.companyList->clear();
    ui.productList->clear();

    if (ui.edtCompanyName->text().size() < 3)
        return;

    List <CompanyRifName> companyList;

    try {

        listCompaniesLike(conn, ui.edtCompanyName->text().toStdString(),
                          companyList);

        for (List <CompanyRifName>::Iterator it(companyList); it.has_current();
             it.next()) {

            CompanyRifName & company = it.get_current();

            QString text;

            text.append(company.rif.c_str());
            text.append(" -- ");
            text.append(company.name.c_str());

            QListWidgetItem * item = new QListWidgetItem(text);
            item->setData(1, QVariant(company.rif.c_str()));

            ui.companyList->addItem(item);
        }
    } catch (const std::exception & e) {

        QString msg = "Se ha capturado la siguiente excepción: ";

        msg.append(e.what());

        QMessageBox::critical(this, "Error", msg);
    }
}

void TempFilterForm::slotCompanySelected(QListWidgetItem * item) {

    if (selectedItem != NULL) {
        selectedItem->setSelected(false);
        selectedItem = NULL;
    }

    ui.btnAddProductToSelectedList->setEnabled(false);
    ui.btnRemoveProductFromSelectedList->setEnabled(false);
    ui.productList->clear();

    List <ProductIdName> productList;

    I(item != NULL);

    try {

        listProductsByCompany(conn, item->data(1).toString().toStdString(),
                              productList);

        for (List <ProductIdName>::Iterator it(productList); it.has_current();
             it.next()) {

            ProductIdName & product= it.get_current();

            QString text;

            text.append(product.name.c_str());
            text.append(" -- ");
            text.append(product.technicalSpecifications.c_str());

            QListWidgetItem * item = new QListWidgetItem(text);
            item->setData(1, QVariant(QString().setNum(product.id)));

            ui.productList->addItem(item);
        }
    } catch (const std::exception & e) {

        QString msg = "Se ha capturado la siguiente excepción: ";

        msg.append(e.what());

        QMessageBox::critical(this, "Error", msg);
    }
}

void TempFilterForm::slotProductSelected(QListWidgetItem * item) {

    if (selectedItem == item)
        return;

    ui.btnAddProductToSelectedList->setEnabled(true);
    ui.btnRemoveProductFromSelectedList->setEnabled(false);

    if (selectedItem != NULL)
        selectedItem->setSelected(false);

    selectedItem = item;
}

void TempFilterForm::slotSelectedProductSelected(QListWidgetItem * item) {

    if (selectedItem == item)
        return;

    ui.btnRemoveProductFromSelectedList->setEnabled(true);
    ui.btnAddProductToSelectedList->setEnabled(false);

    if (selectedItem != NULL)
        selectedItem->setSelected(false);

    selectedItem = item;
}

void TempFilterForm::slotBtnAddProductToSelectedListClicked() {

    long id = selectedItem->data(1).toString().toLong();

    if (productIdSet.insert(id) == NULL)
        return;

    ui.btnRemoveProductFromSelectedList->setEnabled(true);
    ui.btnAddProductToSelectedList->setEnabled(false);

    QListWidgetItem * newItem = new QListWidgetItem(*selectedItem);

    ui.selectedProductList->addItem(newItem);

    selectedItem = newItem;
    newItem->setSelected(true);

    if (not ui.productList->selectedItems().isEmpty())
        ui.productList->selectedItems().at(0)->setSelected(false);

    List<std::string> years;

    getProductionYears(conn, id, years);

    for (List<std::string>::Iterator it(years); it.has_current(); it.next()) {

        const std::string & year = it.get_current();

        unsigned short * ptrQuantity = yearsMap.search(year);

        if (ptrQuantity == NULL) {
            ptrQuantity = yearsMap.insert(year, 0);
        }

        ++(*ptrQuantity);
    }

    updateComboYear();
}

void TempFilterForm::slotBtnRemoveProductFromSelectedList() {

    long id = selectedItem->data(1).toString().toLong();

    productIdSet.remove(id);

    delete selectedItem;
    selectedItem = NULL;

    if (ui.selectedProductList->selectedItems().isEmpty())
        ui.btnRemoveProductFromSelectedList->setEnabled(false);
    else
        selectedItem = ui.selectedProductList->selectedItems().at(0);

    List<std::string> years;

    getProductionYears(conn, id, years);

    for (List<std::string>::Iterator it(years); it.has_current(); it.next()) {

        const std::string & year = it.get_current();

        unsigned short * ptrQuantity = yearsMap.search(year);

        --(*ptrQuantity);

        if (*ptrQuantity == 0) {
            yearsMap.remove(year);
        }
    }

    updateComboYear();
}

void TempFilterForm::slotGraphBuilderExecuted() {

    ui.btnAccept->setEnabled(true);
    ui.btnCancel->setEnabled(true);
    ui.btnAddProductToSelectedList->setEnabled(true);
    ui.btnRemoveProductFromSelectedList->setEnabled(true);
    ui.companyList->setEnabled(true);
    ui.productList->setEnabled(true);
    ui.selectedProductList->setEnabled(true);
    ui.comboStudyType->setEnabled(true);
    ui.comboYear->setEnabled(true);
    ui.spinDownstreamLevels->setEnabled(true);
    ui.spinUpstreamLevels->setEnabled(true);

    waitingPanel.stop();

    Configuration::getInstance().removeDbFile();

    if (process.exitStatus() == QProcess::CrashExit) {
        QMessageBox::critical(
                    this, "Error",
                    "El programa de construcción de red se estrelló");
        return;
    }

    SimType::getInstance().setType(
                StudyType(ui.comboStudyType->currentIndex() - 1));

    SimType::getInstance().setYear(ui.comboYear->currentText().toStdString());

    emit signalFormAccepted();

    close();

}

void TempFilterForm::closeEvent(QCloseEvent *) {
    emit signalClosing();
}
